import java.util.LinkedList;
import java.util.Iterator;
/**
 * Knoten zum Bau von Bumen zur Reprsentation
 * von Organisationsdiagrammen.
 */
class OrganisationsMitglied
{
  // Felder
  /** Der Vorgesetzter dieses Knotens, null = Wurzel 
    * Dieses Feld ist zur einfachen Findung des Vorgesetzten
    * eingefhrt worden.
    */
  private OrganisationsMitglied _vater = null;
  /** Die Untergeordneter linked list weil es beliebig viele sein
   * knnen.
   */
  private LinkedList<OrganisationsMitglied> _kinder = null;
  
  /** Die Person */
  private Person _person = null;
  
  
  // Konstruktor
  /** Konstruktor
   * Erzeugt einen Knoten (Position in der Organisation)
   * mit p als Positionsinhaber.
   * @param p Die Person.
   */
  public OrganisationsMitglied(Person p) {
    _person = p;
  }
  
  
  // Zugriff auf die Felder
  /** Gibt den Vorgesetzten dieser Position zurck, null falls Wurzel. */
  public OrganisationsMitglied vater() { return _vater; }
  
  /** Gibt einen Iterator auf die Untergebenen zurck */
  public Iterator<OrganisationsMitglied> kinderIterator() {
    if (_kinder == null) { return null; }
    else { return _kinder.iterator(); }
  }
  
  /** Fgt ein Untergebenen als letztes Kind ein.
   * Die Vater Beziehung des Kindes wird auch gesetzt.
   * @param u Der Knoten der zum letzten Kind wird.
   */
  public void addKind(OrganisationsMitglied u) {
    if (u != null) {
      if (_kinder == null) { _kinder =
         new LinkedList<OrganisationsMitglied>(); }
      _kinder.addLast(u);
      u._vater = this;
    } 
  }
  
  /** Fgt ein Untergebenen an i-ter stelle ein.
   * Die Vater Beziehung des Kindes wird auch gesetzt.
   * @param u Der Knoten der zum i-ten Kind wird.
   * @param i Die Stelle an welches das Kind eingefgt werden soll.
   * Hat der Knoten weniger wie i Kinder so wird das Kind am Ende
   * angehngt.
   */
  public void addKind(OrganisationsMitglied u, int i) {
    if (u != null) {
      if (_kinder == null) { _kinder =
         new LinkedList<OrganisationsMitglied>(); }
      _kinder.add(i, u);
      u._vater = this;
    } 
  } 
}

/** Klasse zur Speicherung von Personen,
 * die in die Knoten eingefgt werden. 
 */
class Person
{
  private String _famName;
  private String _vorName;
  private int _gehalt;
  
  public Person(String n, String v) {
    _famName = n;
    _vorName = v;
  }
  
  public String toString() {
   return "Name: " + _vorName + _famName;
  }
}